---@diagnostic disable: missing-fields
local M = {
  'hrsh7th/nvim-cmp',
  event = 'InsertEnter',
  dependencies = {
    'hrsh7th/cmp-nvim-lsp',
    'hrsh7th/cmp-buffer',
    'hrsh7th/cmp-path',
    'onsails/lspkind.nvim',
    'saadparwaiz1/cmp_luasnip',
  },
}

function M.config()
  local cmp = require('cmp')
  local kind_func = require('lspkind').cmp_format({})

  cmp.setup({
    -- documentation = true,
    sources = {
      { name = 'nvim_lsp' },
      { name = 'nvim_lua' },
      { name = 'buffer' },
      { name = 'path' },
      { name = 'luasnip' },
      -- { name = 'nvim_lsp_signature_help' },
    },
    formatting = {
      format = function(entry, vim_item)
        local modif = kind_func(entry, vim_item)
        -- folders don't exist on UNIX!
        modif.kind = string.gsub(modif.kind, 'Folder', 'Directory')

        local max_len = 30
        local content = modif.abbr
        if #content > max_len then
           modif.abbr = vim.fn.strcharpart(content, 0, max_len) .. '...'
        end
        return modif
      end,
    },
    window = {
      -- completion = cmp.config.window.bordered(),
      -- documentation = cmp.config.window.bordered(),
    },
    mapping = cmp.mapping.preset.insert({
      ['<C-b>'] = cmp.mapping.scroll_docs(-4),
      ['<C-f>'] = cmp.mapping.scroll_docs(4),
      ['<C-Space>'] = cmp.mapping.complete({}),
      ['<C-e>'] = cmp.mapping.abort(),
      ['<CR>'] = cmp.mapping.confirm({ select = false }),
    }),
    snippet = {
      expand = function(args)
        require('luasnip').lsp_expand(args.body)
      end,
    },
  })

  cmp.setup.filetype({ 'sql', 'mysql', 'plsql' }, {
    sources = {
      { name = 'vim-dadbod-completion' },
      { name = 'buffer' },
    },
  })

  --
  -- Use buffer source for `/` and `?` (if you enabled `native_menu`, this won't work anymore).
  -- cmp.setup.cmdline({ '/', '?' }, {
  --   mapping = cmp.mapping.preset.cmdline(),
  --   sources = {
  --     { name = 'buffer' },
  --   },
  -- })
  -- -- Use cmdline & path source for ':' (if you enabled `native_menu`, this won't work anymore).
  -- cmp.setup.cmdline(':', {
  --   mapping = cmp.mapping.preset.cmdline(),
  --   sources = cmp.config.sources({
  --     { name = 'path' },
  --   }, {
  --     { name = 'cmdline' },
  --   }),
  -- })
end

return M
